XHYBRID_VALUES = {
    "Today's Energy": 8.0,
    "Battery Current": 14.0,
    "Month's Energy": 19.0,
    "Battery Power": 15,
    "Battery Remaining Capacity": 17,
    "Battery Temperature": 16,
    "Battery Voltage": 13,
    "EPS Current": 54,
    "EPS Frequency": 56,
    "EPS Power": 55,
    "EPS Voltage": 53,
    "Exported Power": 10,
    "Grid Frequency": 50,
    "Inverter Temperature": 7,
    "Network Voltage": 5,
    "Output Current": 4,
    "PV1 Current": 0,
    "PV1 Power": 11,
    "PV1 Voltage": 2,
    "PV2 Current": 1,
    "PV2 Power": 12,
    "PV2 Voltage": 3,
    "Power Now": 6,
    "Total Energy": 9,
    "Grid Exported Energy": 41,
    "Grid Imported Energy": 42,
}


X3_VALUES = {
    "PV1 Current": 0,
    "PV2 Current": 1,
    "PV1 Voltage": 2,
    "PV2 Voltage": 3,
    "Output Current Phase 1": 4,
    "Network Voltage Phase 1": 5,
    "AC Power": 6,
    "Inverter Temperature": 7,
    "Today's Energy": 8,
    "Total Energy": 9,
    "Exported Power": 10,
    "PV1 Power": 11,
    "PV2 Power": 12,
    "Battery Voltage": 13,
    "Battery Current": 14,
    "Battery Power": 15,
    "Battery Temperature": 16,
    "Battery Remaining Capacity": 21,
    "Total Feed-in Energy": 41,
    "Total Consumption": 42,
    "Output Current Phase 2": 46,
    "Output Current Phase 3": 47,
    "Network Voltage Phase 2": 48,
    "Network Voltage Phase 3": 49,
    "Grid Frequency Phase 1": 50,
    "Grid Frequency Phase 2": 51,
    "Grid Frequency Phase 3": 52,
    "Power Now Phase 1": 43,
    "Power Now Phase 2": 44,
    "Power Now Phase 3": 45,
    "EPS Voltage": 53,
    "EPS Current": 54,
    "EPS Power": 55,
    "EPS Frequency": 56,
}

X3_HYBRID_VALUES = {
    "PV1 Current": 0,
    "PV2 Current": 0,
    "PV1 Voltage": 0,
    "PV2 Voltage": 0,
    "Output Current Phase 1": 0.9,
    "Network Voltage Phase 1": 234,
    "AC Power": 3189,
    "Inverter Temperature": 42,
    "Today's Energy": 15.2,
    "Total Energy": 27,
    "Exported Power": -25,
    "PV1 Power": 0,
    "PV2 Power": 0,
    "Battery Voltage": 210.3,
    "Battery Current": -15.7,
    "Battery Power": -3321,
    "Battery Temperature": 24,
    "Battery Remaining Capacity": 68,
    "Total Feed-in Energy": 2.03,
    "Total Consumption": 23.41,
    "Output Current Phase 2": 5.4,
    "Output Current Phase 3": 6.9,
    "Network Voltage Phase 2": 250,
    "Network Voltage Phase 3": 251.9,
    "Grid Frequency Phase 1": 50.01,
    "Grid Frequency Phase 2": 50.01,
    "Grid Frequency Phase 3": 50.01,
    "Power Now Phase 1": 123,
    "Power Now Phase 2": 1344,
    "Power Now Phase 3": 1722,
    "EPS Voltage": 0,
    "EPS Current": 0,
    "EPS Power": 0,
    "EPS Frequency": 0,
}

X3V34_HYBRID_VALUES = {
    "Network Voltage Phase 1": 246.8,
    "Network Voltage Phase 2": 249,
    "Network Voltage Phase 3": 250.8,
    "Output Current Phase 1": 1.3,
    "Output Current Phase 2": 1.4,
    "Output Current Phase 3": 1,
    "Power Now Phase 1": 266,
    "Power Now Phase 2": 284,
    "Power Now Phase 3": 136,
    "PV1 Voltage": 537.7,
    "PV2 Voltage": 463,
    "PV1 Current": 1.7,
    "PV2 Current": 0,
    "PV1 Power": 958,
    "PV2 Power": 0,
    "Total PV Energy": 1731.9,
    "Today's PV Energy": 16.4,
    "Grid Frequency Phase 1": 50.03,
    "Grid Frequency Phase 2": 50.03,
    "Grid Frequency Phase 3": 50.03,
    "Total Energy": 1483.3,
    "Today's Energy": 10.3,
    "Battery Voltage": 229.3,
    "Battery Current": 0.9,
    "Battery Power": 229,
    "Battery Temperature": 22,
    "Battery Remaining Capacity": 99,
    "Total Battery Discharge Energy": 706.2,
    "Today's Battery Discharge Energy": 4.3,
    "Battery Remaining Energy": 12.5,
    "Total Battery Charge Energy": 814.2,
    "Today's Battery Charge Energy": 9.1,
    "Exported Power": -56,
    "Total Feed-in Energy": 173.72,
    "Total Consumption": 598.77,
    "AC Power": 686,
    "EPS Frequency": 0,
    "EPS Total Energy": 0.6,
}

X3V34_HYBRID_VALUES_NEGATIVE_POWER = {
    "Network Voltage Phase 1": 236.4,
    "Network Voltage Phase 2": 243.1,
    "Network Voltage Phase 3": 238.6,
    "Output Current Phase 1": -7.3,
    "Output Current Phase 2": -7.2,
    "Output Current Phase 3": -7.2,
    "Power Now Phase 1": -1738,
    "Power Now Phase 2": -1767,
    "Power Now Phase 3": -1729,
    "PV1 Voltage": 0,
    "PV2 Voltage": 0,
    "PV1 Current": 0,
    "PV2 Current": 0,
    "PV1 Power": 0,
    "PV2 Power": 0,
    "Total PV Energy": 9615.2,
    "Today's PV Energy": 9.8,
    "Grid Frequency Phase 1": 49.98,
    "Grid Frequency Phase 2": 49.98,
    "Grid Frequency Phase 3": 49.98,
    "Total Energy": 8478.7,
    "Today's Energy": 8.4,
    "Battery Voltage": 204.6,
    "Battery Current": 25,
    "Battery Power": 5117,
    "Battery Temperature": 24,
    "Battery Remaining Capacity": 20,
    "Total Battery Discharge Energy": 2469.6,
    "Today's Battery Discharge Energy": 4,
    "Battery Remaining Energy": 2.6,
    "Total Battery Charge Energy": 2887.6,
    "Today's Battery Charge Energy": 6,
    "Exported Power": -5744,
    "Total Feed-in Energy": 3593.54,
    "Total Consumption": 945.63,
    "AC Power": -5234,
    "EPS Frequency": 0,
    "EPS Total Energy": 2.1,
}

X3V34_HYBRID_VALUES_EPS_MODE = {
    "Network Voltage Phase 1": 0,
    "Network Voltage Phase 2": 0,
    "Network Voltage Phase 3": 0,
    "Output Current Phase 1": 0,
    "Output Current Phase 2": 0,
    "Output Current Phase 3": 0,
    "Power Now Phase 1": 0,
    "Power Now Phase 2": 0,
    "Power Now Phase 3": 0,
    "PV1 Voltage": 486.4,
    "PV2 Voltage": 446.6,
    "PV1 Current": 5.5,
    "PV2 Current": 0,
    "PV1 Power": 2678,
    "PV2 Power": 0,
    "Total PV Energy": 8378.8,
    "Today's PV Energy": 26,
    "Grid Frequency Phase 1": 0,
    "Grid Frequency Phase 2": 0,
    "Grid Frequency Phase 3": 0,
    "Total Energy": 7387.5,
    "Today's Energy": 17.8,
    "Battery Voltage": 228.6,
    "Battery Current": -1.00,
    "Battery Power": -242,
    "Battery Temperature": 27,
    "Battery Remaining Capacity": 98,
    "Total Battery Discharge Energy": 2124,
    "Today's Battery Discharge Energy": 6.1,
    "Battery Remaining Energy": 12.4,
    "Total Battery Charge Energy": 2501.1,
    "Today's Battery Charge Energy": 11.9,
    "Exported Power": 0,
    "Total Feed-in Energy": 3174.83,
    "Total Consumption": 844.58,
    "AC Power": 0,
    "EPS Frequency": 50,
    "EPS Total Energy": 1.7,
}

X3_HYBRID_G4_VALUES = {
    "Grid 1 Voltage": 238.3,
    "Grid 2 Voltage": 239.7,
    "Grid 3 Voltage": 237.7,
    "Grid 1 Current": 1.0,
    "Grid 2 Current": 1.1,
    "Grid 3 Current": 1.1,
    "Grid 1 Power": 33.0,
    "Grid 2 Power": 81.0,
    "Grid 3 Power": 76.0,
    "PV1 Voltage": 248.3,
    "PV2 Voltage": 230.5,
    "PV1 Current": 0.1,
    "PV2 Current": 0.2,
    "PV1 Power": 45.0,
    "PV2 Power": 58.0,
    "Grid 1 Frequency": 50.01,
    "Grid 2 Frequency": 50.01,
    "Grid 3 Frequency": 50.01,
    "Run mode text": "Normal",
    "EPS 1 Voltage": 0.0,
    "EPS 2 Voltage": 0.0,
    "EPS 3 Voltage": 0.0,
    "EPS 1 Current": 0.0,
    "EPS 2 Current": 0.0,
    "EPS 3 Current": 0.0,
    "EPS 1 Power": 0.0,
    "EPS 2 Power": 0.0,
    "EPS 3 Power": 0.0,
    "Grid Power ": -152.0,
    "Battery Current": 0.0,
    "Battery Power": 0.0,
    "Load/Generator Power": 342.0,
    "Radiator Temperature": 23.0,
    "Yield total": 4.4,
    "Yield today": 0.1,
    "Battery Discharge Energy total": 2.4,
    "Battery Charge Energy total": 1.0,
    "Battery Discharge Energy today": 0.0,
    "Battery Charge Energy today": 0.0,
    "PV Energy total": 3.4,
    "EPS Energy total": 0.0,
    "EPS Energy today": 0.0,
    "Feed-in Energy": 0.0,
    "Consumed Energy": 20.09,
    "Feed-in Energy total": 0.0,
    "Consumed Energy total": 2.09,
    "Battery Remaining Capacity": 30.0,
    "Battery Temperature": 22.0,
    "Battery Remaining Energy": 3.3,
    "Battery mode": 0.0,
    "Battery mode text": "Self Use Mode",
    "Battery Voltage": 234.6,
}

X3_MICPRO_G2_VALUES = {
    "Grid 1 Voltage": 237.1,
    "Grid 2 Voltage": 234.5,
    "Grid 3 Voltage": 237.8,
    "Grid 1 Current": 4.1,
    "Grid 2 Current": 4.1,
    "Grid 3 Current": 4.1,
    "Grid 1 Power": 1018.0,
    "Grid 2 Power": 992.0,
    "Grid 3 Power": 970.0,
    "PV1 Voltage": 174.1,
    "PV2 Voltage": 174.5,
    "PV3 Voltage": 0.0,
    "PV1 Current": 8.8,
    "PV2 Current": 8.7,
    "PV3 Current": 0.0,
    "PV1 Power": 1544.0,
    "PV2 Power": 1531.0,
    "PV3 Power": 0.0,
    "Grid 1 Frequency": 49.98,
    "Grid 2 Frequency": 49.99,
    "Grid 3 Frequency": 49.94,
    "Run Mode": "Normal",
    "Total Yield": 795.7,
    "Daily Yield": 20.0,
    "Feed-in Power ": 2707.0,
    "Total Feed-in Energy": 657.24,
    "Total Consumption": 307.33,
}

X1_VALUES = {
    "PV1 Current": 0,
    "PV2 Current": 1,
    "PV1 Voltage": 2,
    "PV2 Voltage": 3,
    "Output Current": 4,
    "Network Voltage": 5,
    "AC Power": 6,
    "Inverter Temperature": 7,
    "Today's Energy": 8,
    "Total Energy": 9,
    "Exported Power": 10,
    "PV1 Power": 11,
    "PV2 Power": 12,
    "Battery Voltage": 13,
    "Battery Current": 14,
    "Battery Power": 15,
    "Battery Temperature": 16,
    "Battery Remaining Capacity": 21,
    "Total Feed-in Energy": 41,
    "Total Consumption": 42,
    "Grid Frequency": 50,
    "Power Now": 43,
    "EPS Voltage": 53,
    "EPS Current": 54,
    "EPS Power": 55,
    "EPS Frequency": 56,
}

X1_MINI_VALUES = {
    "PV1 Current": 0,
    "PV2 Current": 1,
    "PV1 Voltage": 2,
    "PV2 Voltage": 3,
    "Output Current": 4,
    "Network Voltage": 5,
    "AC Power": 6,
    "Inverter Temperature": 7,
    "Today's Energy": 8,
    "Total Energy": 9,
    "Exported Power": 10,
    "PV1 Power": 11,
    "PV2 Power": 12,
    "Total Feed-in Energy": 41,
    "Total Consumption": 42,
    "Power Now": 43,
    "Grid Frequency": 50,
}

X1_MINI_VALUES_V34 = {
    "Network Voltage": 231.0,
    "Output Current": 1.5,
    "AC Power": 349,
    "PV1 Voltage": 60.9,
    "PV2 Voltage": 0,
    "PV1 Current": 5.8,
    "PV2 Current": 0,
    "PV1 Power": 359,
    "PV2 Power": 0,
    "Grid Frequency": 50.0,
    "Total Energy": 347.4,
    "Today's Energy": 1.2,
    "Total Feed-in Energy": 251.8,
    "Total Consumption": 0,
    "Power Now": 0,
    "Inverter Temperature": 41,
}


X1_MINI_VALUES_V34_VER3 = {
    "Network Voltage": 234.8,
    "Output Current": 0.7,
    "AC Power": 144.0,
    "PV1 Voltage": 96.6,
    "PV2 Voltage": 0.0,
    "PV1 Current": 1.6,
    "PV2 Current": 0.0,
    "PV1 Power": 154.0,
    "PV2 Power": 0.0,
    "Grid Frequency": 49.94,
    "Total Energy": 6.7,
    "Today's Energy": 3.1,
    "Total Feed-in Energy": 2.5,
    "Total Consumption": 0.0,
    "Power Now": 0.0,
    "Inverter Temperature": 39.0,
}

X1_SMART_VALUES = {
    "Network Voltage": 239.6,
    "Output Current": 12.6,
    "AC Power": 2956,
    "PV1 Voltage": 432.9,
    "PV2 Voltage": 233.8,
    "PV1 Current": 4.1,
    "PV2 Current": 5.7,
    "PV1 Power": 1777,
    "PV2 Power": 1336,
    "Grid Frequency": 60.21,
    "Total Energy": 121.5,
    "Today's Energy": 13.7,
    "Inverter Temperature": 43,
    "Exported Power": 10,
    "Total Feed-in Energy": 0.87,
    "Total Consumption": 81.84,
}

X1_BOOST_VALUES = {
    "AC Voltage": 239.6,
    "AC Output Current": 12.6,
    "AC Output Power": 2956,
    "PV1 Voltage": 432.9,
    "PV2 Voltage": 233.8,
    "PV1 Current": 4.1,
    "PV2 Current": 5.7,
    "PV1 Power": 1777,
    "PV2 Power": 1336,
    "AC Frequency": 60.21,
    "Total Generated Energy": 121.5,
    "Today's Generated Energy": 13.7,
    "Inverter Temperature": 43,
    "Exported Power": 10,
    "Total Export Energy": 0.87,
    "Total Import Energy": 81.84,
}

X1_BOOST_VALUES_OVERFLOWN = {
    "AC Voltage": 237.4,
    "AC Output Current": 6.7,
    "AC Output Power": 1581,
    "PV1 Voltage": 397.3,
    "PV2 Voltage": 233.8,
    "PV1 Current": 4.1,
    "PV2 Current": 5.7,
    "PV1 Power": 1628,
    "PV2 Power": 1336,
    "AC Frequency": 49.96,
    "Total Generated Energy": 14055.6,
    "Today's Generated Energy": 1.3,
    "Inverter Temperature": 36,
    "Exported Power": 1444,
    "Total Export Energy": 11348.18,
    "Total Import Energy": 1850.05,
}

QVOLTHYBG33P_VALUES = {
    "Network Voltage Phase 1": 221.4,
    "Network Voltage Phase 2": 223.8,
    "Network Voltage Phase 3": 225.1,
    "Output Current Phase 1": 1.1,
    "Output Current Phase 2": 1.0,
    "Output Current Phase 3": 1.2,
    "Power Now Phase 1": 162.0,
    "Power Now Phase 2": 136.0,
    "Power Now Phase 3": 146.0,
    "AC Power": 444.0,
    "PV1 Voltage": 566.2,
    "PV2 Voltage": 568.2,
    "PV1 Current": 1.8,
    "PV2 Current": 1.7,
    "PV1 Power": 1050.0,
    "PV2 Power": 977.0,
    "Grid Frequency Phase 1": 50.02,
    "Grid Frequency Phase 2": 50.01,
    "Grid Frequency Phase 3": 50.02,
    "Inverter Operation mode": "Normal",
    "Exported Power": -7.0,
    "Battery Voltage": 323.4,
    "Battery Current": 5.0,
    "Battery Power": 1616.0,
    "Power Now": 451.0,
    "Total Energy": 219.0,
    "Total Battery Discharge Energy": 73.8,
    "Total Battery Charge Energy": 90.4,
    "Today's Battery Discharge Energy": 0.0,
    "Today's Battery Charge Energy": 8.1,
    "Total PV Energy": 231.6,
    "Today's Energy": 11.8,
    "Total Feed-in Energy": 107.94,
    "Total Consumption": 145.44,
    "Today's Feed-in Energy": 1.66,
    "Today's Consumption": 4.66,
    "Battery Remaining Capacity": 95.0,
    "Battery Temperature": 35.0,
    "Battery Remaining Energy": 8.8,
    "Battery Operation mode": "Self Use Mode",
}

X1_HYBRID_G4_VALUES = {
    "AC voltage R": 247.0,
    "AC current": 1.1,
    "AC power": 237.0,
    "Grid frequency": 49.86,
    "PV1 voltage": 254.4,
    "PV2 voltage": 259.1,
    "PV1 current": 1.1,
    "PV2 current": 1.2,
    "PV1 power": 299.0,
    "PV2 power": 329.0,
    "On-grid total yield": 398.6,
    "On-grid daily yield": 2.2,
    "Battery voltage": 239.30,
    "Battery current": 2.20,
    "Battery power": 541.0,
    "Battery temperature": 26.0,
    "Battery SoC": 82.0,
    "Inverter Temperature": 30.0,
    "Grid power": 1.0,
    "Total feed-in energy": 286.7,
    "Total consumption": 6.2,
}

X3_ULTRA_VALUES = {
    "Grid 1 Voltage": 232.8,
    "Grid 2 Voltage": 233.8,
    "Grid 3 Voltage": 232.0,
    "Grid 1 Current": 2.0,
    "Grid 2 Current": 1.9,
    "Grid 3 Current": 1.9,
    "Grid 1 Power": 216.0,
    "Grid 2 Power": 249.0,
    "Grid 3 Power": 199.0,
    "PV1 Voltage": 0.0,
    "PV2 Voltage": 249.8,
    "PV3 Voltage": 245.4,
    "PV1 Current": 0.0,
    "PV2 Current": 6.5,
    "PV3 Current": 10.5,
    "PV1 Power": 0.0,
    "PV2 Power": 1629.0,
    "PV3 Power": 2592.0,
    "Grid 1 Frequency": 50.0,
    "Grid 2 Frequency": 50.0,
    "Grid 3 Frequency": 50.0,
    "Run mode text": "Normal",
    "EPS 1 Voltage": 0.0,
    "EPS 2 Voltage": 0.0,
    "EPS 3 Voltage": 0.0,
    "EPS 1 Current": 0.0,
    "EPS 2 Current": 0.0,
    "EPS 3 Current": 0.0,
    "EPS 1 Power": 0.0,
    "EPS 2 Power": 0.0,
    "EPS 3 Power": 0.0,
    "Grid Power ": 52.0,
    "Battery 1 Voltage": 213.8,
    "Battery 2 Voltage": 212.9,
    "Battery 1 Current": 9.6,
    "Battery 2 Current": 7.4,
    "Battery 1 Power": 2057.0,
    "Battery 2 Power": 1595.0,
    "Battery 1 Remaining Capacity": 28.0,
    "Battery 2 Remaining Capacity": 28.0,
    "Battery 1 Temperature": 13.0,
    "Battery 2 Temperature": 12.0,
    "Load/Generator Power": -52.0,
    "Radiator Temperature": 34.0,
    "Yield total": 198.4,
    "Yield today": 2.4,
    "Battery Discharge Energy total": 109.2,
    "Battery Charge Energy total": 122.8,
    "Battery Discharge Energy today": 0.2,
    "Battery Charge Energy today": 2.7,
    "PV Energy total": 122.9,
    "EPS Energy total": 0.0,
    "EPS Energy today": 0.0,
    "Feed-in Energy total": 27.71,
    "Grid Consumed Energy total": 128.12,
    "Feed-in Energy today": 0.1,
    "Grid Consumed Energy today": 8.6,
    "Battery Remaining Capacity": 28.0,
    "Battery Remaining Energy": 1.7,
    "Inverter Power": 66.4,
}
